"""
vaultpass

VaultPass: A CyberArk Search Script. Search for accounts and retrieve their passwords in the CyberArk elevated access management application. You can only retrieve accounts that you are able to view
"""

__version__ = "1.0.15"
__author__ = 'Jack Stewart'
__credits__ = 'University of Michigan'

./vaultpass.py

